/**********************************************************************
 *
 * GEOS - Geometry Engine Open Source
 * http://geos.osgeo.org
 *
 * Copyright (C) 2007 Refractions Research Inc.
 *
 * This is free software; you can redistribute and/or modify it under
 * the terms of the GNU Lesser General Public Licence as published
 * by the Free Software Foundation.
 * See the COPYING file for more information.
 *
 **********************************************************************/
#ifndef GEOS_VERSION_H_INCLUDED
#define GEOS_VERSION_H_INCLUDED

#ifndef GEOS_VERSION_MAJOR
#define GEOS_VERSION_MAJOR 3
#endif

#ifndef GEOS_VERSION_MINOR
#define GEOS_VERSION_MINOR 9
#endif

#ifndef GEOS_VERSION_PATCH
#define GEOS_VERSION_PATCH 1dev
#endif

#ifndef GEOS_VERSION
#define GEOS_VERSION "3.9.1dev"
#endif

#ifndef GEOS_JTS_PORT
#define GEOS_JTS_PORT "1.17.0"
#endif

#endif // GEOS_VERSION_H_INCLUDED
